﻿/*
   Copyright 2009 adamrocker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

// Preview.java
package com.adamrocker.android.camezon;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.Result;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Point;
import android.graphics.Rect;
import android.hardware.Camera;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import static com.adamrocker.android.camezon.CameZonActivity.TAG;

//미리보기 및 데이터 분석을 수행
class Preview extends SurfaceView implements SurfaceHolder.Callback {

    // 자동 초점 사진을 분석할 수
    private static final int TRY_PREVIEW = 15;

    private CameZonActivity mCamezon;
    private Camera mCamera;
    private FinderView mFinder;
    private SurfaceHolder mHolder;
    private Rect mFramingRect;//분석 영역
    private Point mResolution;//이미지의 가로 세로 픽셀
    private Decoder mDecoder;// 바코드 디코더
    private ResultDialog mResultDialog;
    private int previewCount;
    private boolean isPreviewing = false;

    public Preview(Context context, AttributeSet attrs) {
        super(context, attrs);
        Log.i(TAG, "new Preview");

        mDecoder = new Decoder();
        mResultDialog = new ResultDialog(context);
        mCamezon = (CameZonActivity) context;

        mHolder = getHolder();
        mHolder.addCallback(this);
        mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
    }

    public void surfaceCreated(SurfaceHolder holder) {
        Log.i(TAG, "surfaceCreated");
        // surface이 만들어지는과 함께 카메라 열기
        mCamera = Camera.open();
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        Log.i(TAG, "surfaceDestroyed");
        if (mCamera != null && isPreviewing) {
            isPreviewing = false;
            // surface의 제거시 카메라를 풀어 놓는다
            closeCamera();
        }
    }

    public void surfaceChanged(SurfaceHolder holder, int format,
                               int w, int h) {
        Log.i(TAG, "surfaceChanged / w:" + w + ", h:" + h);

        if (mCamera != null) {
            if (isPreviewing) {
                mCamera.stopPreview();
            }

            // 카메라 매개 변수 설정
            Camera.Parameters parameters = mCamera.getParameters();
            parameters.setPreviewSize(w, h);
            mCamera.setParameters(parameters);
            // SDK1.5에서 setPreviewDisplay는 IOException을 throw한다
            try {
				mCamera.setPreviewDisplay(holder);
			} catch (Exception e) {
				e.printStackTrace();
			}

            //카메라 미리보기 시작
            mCamera.startPreview();
            isPreviewing = true;
            requestPreview();
        }

        refreshScreenResolution();
        refreshFrameRect();
    }

    void setFinder(FinderView finder) {
        mFinder = finder;
    }

    public void closeCamera() {
        if (mCamera != null) {
            mCamera.stopPreview();
            mCamera.release();
            mCamera = null;
        }
    }

    /**
     *측정기 영역을 설정
     */
    private void refreshFrameRect() {
        if (mFinder != null) {
            mFinder.setFramingRect(getFrameRect());
        }
    }

    // 측정기의 영역을 생성
    private Rect getFrameRect() {
        Log.i(TAG, "getFramingRect");
        if (mFramingRect == null) {
            int size = 
                ((mResolution.x < mResolution.y) ? 
                 mResolution.x : mResolution.y) * 3 / 4;
            int leftOffset = (mResolution.x - size) / 2;
            int topOffset = (mResolution.y - size) / 2;
            mFramingRect = 
                new Rect(leftOffset, topOffset, leftOffset + size,
                         topOffset + size);
        }
        return mFramingRect;
    }

    // 레이아웃 해상도를 받음
    // Decoder에 전달하는 1 차원 배열의 가로 세로 픽셀이된다.
    private void refreshScreenResolution() {
        int w = getWidth();
        int h = getHeight();
        Log.i(TAG, "refreshScreenResolution(w:" + w + ", h:" + h);
        mResolution = new Point(w, h);
    }

    // 자동 초점 시작
    void requestAutoFocus() {
        Log.i(TAG, "requestAutoFocus");
        if (mCamera != null) {
            mCamera.autoFocus(autoFocusCallback);
        }
    }

    // 미리보기 검색
    private void requestPreview() {
        Log.i(TAG, "requestPreview");
        if (mCamera != null) {
            mCamera.setPreviewCallback(previewCallback);
        }
    }

    // PreviewCallback과 AutoFocusCallback 상호 재귀 호출의 형태가된다
    // 실제로 setPreviewCallback과 autoFocus 콜백 함수 setter에서 실현하고있다
    private final Camera.PreviewCallback previewCallback = 
        new Camera.PreviewCallback() {
          public void onPreviewFrame(byte[] data, Camera camera) {
              Log.i(TAG, "onPreviewFrame");
              camera.setPreviewCallback(null);
              Result result = 
                  mDecoder.decode(data, mResolution.x,
                                  mResolution.y,
                                  getFrameRect());
            if (result != null) {
                previewCount = 0;
                BarcodeFormat code = result.getBarcodeFormat();
                Log.i(TAG, "preview success(" + code + ", " 
                      + result.getText() + ")");

                // EAN13 형식의 바코드에서만 계속 작업을 수행
                if (code == BarcodeFormat.EAN_13) {
                    Bitmap img = mDecoder.getResultBitmap0();
                    mResultDialog.set(img, result);
                    mResultDialog.show();
                } else {
                    // EAN13 형식 의외의 바코드 경우
                    // 경고 대화 상자 표시
                    mCamezon.showIncorrectDialog(code.toString());
                }
            } else {
                Log.i(TAG, "preview failed");
                if (previewCount < TRY_PREVIEW) {
                    Log.i(TAG, "retry preview");
                    previewCount++;
                    requestPreview();
                } else {
                    Log.i(TAG, "give up preview");
                    previewCount = 0;
                    // 해석 불가이므로 다시 자동 초점
                    // 이미지 검색 재등록
                    requestAutoFocus();
                }
            }
        }
    };

    // PreviewCallback과 AutoFocusCallback 상호 부르는 형태가된다
    private final Camera.AutoFocusCallback autoFocusCallback = 
        new Camera.AutoFocusCallback() {
            public void onAutoFocus(boolean success, Camera camera) {
                Log.i(TAG, "onAutoFocus");
                camera.autoFocus(null);
                // 미리보기를 보유하고 바코드 분석
                requestPreview();
            }
        };
}
